from ._base import Base


class Group(Base):
    home_path = 'groups'

    def data(self, name, inventory_id, description=None, variables={}):
        description = description if description is not None else ''
        return {"name": name, "description": description, "inventory": inventory_id, "variables": variables}



from ._base import Base


class Host(Base):
    home_path = 'hosts'

    def data(self, name, inventory_id, description=None, variables={}):
        description = description if description is not None else ''
        return {
            "name": name,
            "description": description,
            "inventory": inventory_id,
            "variables": variables,
            "instance_id": "",
            "enable": True
        }



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
import yaml
import src.aap as aap
import tests.vars as vars
import cndprint


cnd_print = cndprint.CndPrint(level="Info", uuid="MambaTest", silent_mode=False)
host = ''
with description('CndApp') as self:
    with before.each:
        unstub()
        self.instance = aap.cnd_aap.CndAap(vars.host, _print=vars._print)
        self.instance = self.instance.group

    with context("__init__"):
        with it("shoud get an instance"):
            expect(isinstance(self.instance, aap._group.Group)).to(equal(True))

    with context("data"):
        with it("shoud support the right field"):
            result = self.instance.data("name", "inventory_id", description="description", variables={"a": "b"})
            expected_result = {
                'name': 'name',
                'description': 'description',
                'inventory': 'inventory_id',
                'variables': {'a': 'b'}
            }
            expect(result).to(equal(expected_result))



import requests
import cndprint
import urllib3
import json
urllib3.disable_warnings()


class Adapter:
    prefix = "/api/v2/"
    home_path = ''

    def __init__(self, host, creds={"username": None, "password": None}, _print=cndprint.CndPrint()):
        self._host = host
        self._creds = creds
        self._print = _print
        self.default_headers = {"Content-Type": "application/json"}

    def api(self):
        result = self._query('me')
        return result["content"]

    def display_log(self, url, method, response):
        self._print.trace_v(f'Url: {url}')
        self._print.trace_v(f'Method: {method}')
        self._print.trace_v(f'Status Code: {response.status_code}')
        self._print.trace_v(f'Response Content: {response.content}')

    def _query(self, url, method="get", json_data=None, retry=False, retry_count=3):
        if method not in ['get', 'post', 'delete', 'patch', 'put']:
            raise AttributeError("(CND) Method not allowed")
        url = url.replace(self.prefix, '')
        full_url = f"{self._host}{self.prefix}{url}"
        self.kwargs = {}
        if json_data is not None:
            self.kwargs["json"] = json_data
        response = getattr(requests, method)(
            full_url,
            headers=self.default_headers,
            auth=(self._creds["username"], self._creds["password"]),
            verify=False,
            **self.kwargs
        )
        self.display_log(full_url, method, response)
        if response.status_code in [200, 201, 202, 204, 404]:
            return {
                "content": json.loads(response.content),
                "status_code": response.status_code
            }
        self._print.info_e(f'Error with status code: {response.status_code}')
        return False



import cndprint
import yaml
import src.aap as aap


creds = {"username": '', "password": ''}
host = 'https://iamnotexisting.com'
_print = cndprint.CndPrint(level="info", uuid=">> ", silent_mode=True)

def read_file(filename):
    return open(filename).read()
    
def read_yaml_file(filename):
    content = read_file(filename)
    return yaml.safe_load(content)
    
adapter = aap.adapter.Adapter(host, creds=creds, _print=_print)



import requests
import cndprint
import urllib3
import json
from .adapter import Adapter
urllib3.disable_warnings()


class Base:
    prefix = "/api/v2/"
    home_path = ''
    
    def __init__(self, adapter, _print):
        self._adapter = adapter
        self._print = _print

    def data(self, name):
        return {"name": name}

    def find_all(self, next_page=None):
        self._print.log_d(f"find_all {type(self).__name__}")
        url = next_page if next_page is not None else self.home_path
        result = self._adapter._query(url)
        items = result["content"]["results"]
        if result["content"]["next"] is not None:
            items += self.find_all(result["content"]["next"])
        return items

    def destroy(self, id):
        self._print.log_d(f"Find_id {type(self).__name__}")
        result = self._adapter._query(f'{self.home_path}/{id}', method="delete")
        if result["status_code"] == 204:
            return True
        return False

    def find_id(self, name):
        self._print.log_d(f"Find_id {type(self).__name__}: {name}")
        result = self._adapter._query(f'{self.home_path}?search={name}')
        return result["content"]["results"][0]["id"]

    def _create(self, data):
        self._print.log_d(f"Create {type(self).__name__}")
        result = self._adapter._query(f"{self.home_path}", method="post", json_data=data)
        return result["content"]["id"]

    def _update(self, id, data):
        self._print.log_d(f"Update {type(self).__name__}")
        result = self._adapter._query(f"{self.home_path}", method="put", json_data=data)
        return result["content"]["id"]

    def create_or_update(self, data):
        self._print.log_d(f"create_or_update {type(self).__name__}")
        id = self.find_id(data["name"])
        if id is not None:
            return self._update(id, data)
        return self._create(data)



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
from requests_mock import Mocker
import yaml
import requests
import src.aap as aap
import tests.vars as vars


with description('Base') as self:
    
    with before.each:
        unstub()
        self.instance = aap._base.Base(vars.adapter, vars._print)

    with context("__init__"):
        with it("shoud get an instance"):
            expect(isinstance(self.instance, aap._base.Base)).to(equal(True))

#    with context("connect"):
#        with it("shoud return the 'me' information"):
#            with Mocker(real_http=True) as m:
#                result = vars.read_yaml_file("tests/data/base/api.json")
#                m.get(f'{vars.host}/api/v2/me', status_code=200, json=result)
#                result = self.instance.api()
#                expect(result["results"][0]["username"]).to(equal('admin'))
#
#    with context("_query", "_query"):
#        with it("shoud raise an error if method is not correct"):
#            expect(lambda: self.instance._query('', method="wrong_method")).to(raise_error(AttributeError))
#            
#        with it("shoud raise an error if method is not correct"):
#            response = mock({'content': '{}', 'status_code': 500}, spec=requests.Response)
#            when(requests).get(...).thenReturn(response)
#            result = self.instance._query('')
#            expect(result).to(equal(False))

    with context("destroy", "destroy"):
        with it("shoud return True after destroy item"):
            with Mocker(real_http=True) as m:
                m.delete(f'{vars.host}/api/v2//123', status_code=204, json={"content": {}})
                result = self.instance.destroy(123)
                expect(result).to(equal(True))
                    
        with it("shoud return False after destroy item failed"):
            with Mocker(real_http=True) as m:
                m.delete(f'{vars.host}/api/v2//123', status_code=404, json={"content": {}})
                result = self.instance.destroy(123)
                expect(result).to(equal(False))

    with context("_update", "_update"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/base/create.json")
                m.put(f'{vars.host}/api/v2/', status_code=201, json=result)
                result = self.instance._update(id, self.instance.data('Abc'))
                expect(result).to(equal(456))

    with context("_create", "_create"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/base/create.json")
                m.post(f'{vars.host}/api/v2/', status_code=201, json=result)
                result = self.instance._create(self.instance.data('Abc'))
                expect(result).to(equal(456))

    with context("find_id", "find_id"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/base/find.json")
                m.get(f'{vars.host}/api/v2/?search=Org_name', status_code=200, json=result)
                result = self.instance.find_id("Org_name")
                expect(result).to(equal(123))

    with context("find_all", "find_all"):
        with it("shoud return multiple"):
            with Mocker(real_http=True) as m:
                page1 = vars.read_yaml_file("tests/data/base/find_all-page1.json")
                page2 = vars.read_yaml_file("tests/data/base/find_all-page2.json")
                m.get(f'{vars.host}/api/v2/', status_code=200, json=page1)
                m.get(f'{vars.host}/api/v2/?page=2', status_code=200, json=page2)
                result = self.instance.find_all()
                expect(len(result)).to(equal(3))

    with context("create_or_update", "create_or_update"):
        with it("shoud create if find return none"):
            when(self.instance).find_id(...).thenReturn(None)
            when(self.instance)._create(...).thenReturn(1)
            result = self.instance.create_or_update({"name": "Org_name"})
            expect(result).to(equal(1))
            
        with it("shoud update if find return an id"):
            when(self.instance).find_id(...).thenReturn(1)
            when(self.instance)._update(...).thenReturn(2)
            result = self.instance.create_or_update({"name": "Org_name"})
            expect(result).to(equal(2))



from ._base import Base
import time


class Workflow(Base):
    home_path = 'workflow_job_templates'

    def data(self, name, inventory_id, description=None, variables={}):
        description = description if description is not None else ''
        return {"name": name, "description": description, "inventory": inventory_id, "variables": variables}
        
    def _execute(self, name, inventory_id):
        self._print.log_d(f"Execute {type(self).__name__}")
        workflow_job_id = self.find_id(name)
        data = {
          "inventory": inventory_id
        }
        result = self._adapter._query(f"{self.home_path}/{workflow_job_id}/launch/", method="post", json_data=data)
        return result["content"]["workflow_job"]
        
    def _wait(self, workflow_job_id):
        iteration_time = 100
        self._print.log_d(f"Wait {type(self).__name__}")
        finished = None
        while finished is not True and iteration_time > 0:
            iteration_time += -1
            time.sleep(10)
            result = self._adapter._query(f"workflow_jobs/{id}/")
            if result["content"]["finished"] is None:
                continue
            finished = True
        if result["content"]["finished"] is None:
            return None
        return not result["content"]["failed"]

    def execute_and_wait(self, name, inventory_id):
        workflow_job_id = self._execute(name, inventory_id)
        return self._wait(workflow_job_id)



from ._base import Base


class Organization(Base):
    home_path = 'organizations'

    def data(self, name, description=None):
        description = description if description is not None else ''
        return {"name": name, "description": description}



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
from requests_mock import Mocker
import yaml
import requests
import src.aap as aap
import tests.vars as vars


with description('Base') as self:
    
    with before.each:
        unstub()
        self.instance = aap.adapter.Adapter(vars.host, creds=vars.creds, _print=vars._print)

    with context("__init__"):
        with it("shoud set the host, username and password"):
            expect(self.instance._host).to(equal(vars.host))

    with context("connect"):
        with it("shoud return the 'me' information"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/base/api.json")
                m.get(f'{vars.host}/api/v2/me', status_code=200, json=result)
                result = self.instance.api()
                expect(result["results"][0]["username"]).to(equal('admin'))

    with context("_query", "_query"):
        with it("shoud raise an error if method is not correct"):
            expect(lambda: self.instance._query('', method="wrong_method")).to(raise_error(AttributeError))
            
        with it("shoud raise an error if method is not correct"):
            response = mock({'content': '{}', 'status_code': 500}, spec=requests.Response)
            when(requests).get(...).thenReturn(response)
            result = self.instance._query('')
            expect(result).to(equal(False))



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
import yaml
import src.aap as aap
import tests.vars as vars


with description('CndApp') as self:
    with before.each:
        unstub()
        self.instance = aap.cnd_aap.CndAap(vars.host)

    with context("__init__"):
        with it("shoud get an instance"):
            expect(isinstance(self.instance, aap.CndAap)).to(equal(True))
